
//          Copyright Danny Havenith 2009.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

// This file is a minimal adaptation of the template source file that is provided by the lcd smartie 
// project.

#include <cstdio>
#include <string>
#include "LCDS_Plugin.H"
#include "packager.hpp"
#include "serial_port_sink.hpp"

using namespace std;

// this implementation is hard-coded to send all data to COM1.
serial_port_sink sink( "COM1");
packager packager( sink);
string pending_commands;

// +-----------------+
// ! DISPLAYDLL_Init !
// +-----------------+
//! Initializes the plugin.
//! All initialization stuff goes there: opening and configuring IO ports, device detection and probing...
//! Sets ok to 1 if initialization is successful, 0 otherwise.
//! Returns an error message in case of failure, NULL otherwise.
DLL_EXPORT(char *) DISPLAYDLL_Init(LCDS_BYTE size_x,LCDS_BYTE size_y,char *startup_parameters,LCDS_BOOL *ok) {
  *ok=1;
  return NULL;
}

// +-----------------------+
// ! DISPLAYDLL_DriverName !
// +-----------------------+
//! Returns the driver name.
//! The driver name is used by the driver selection GUI to identify the driver.
//! It's good practice to return a proper name and version for the driver so that
//! it can be clearly identified (or else we would have to rely on the plugin name
//! which is subject to modifications).
DLL_EXPORT(char *) DISPLAYDLL_DriverName(void) {
  static char plugin_name[]="SX Wireless LCD v0.1";
  return plugin_name;
}

// +------------------+
// ! DISPLAYDLL_Usage !
// +------------------+
//! Returns plugin usage hints.
//! The usage text describes the main plugin parameters and how they are assembled
//! in the parameter string (syntax indications).
/*DLL_EXPORT(char *) DISPLAYDLL_Usage(void) {
  static char plugin_usage[]="my_plugin usage";
  return plugin_usage;
}*/

// +------------------------------+
// ! DISPLAYDLL_DefaultParameters !
// +------------------------------+
//! Returns the plugin default parameters.
//! The default parameters string is used as parameter string after the driver
//! has been initialized.
/*DLL_EXPORT(char *) DISPLAYDLL_DefaultParameters(void) {
  static char plugin_default_parameters[]="default";
  return plugin_default_parameters;
}*/

// +------------------------+
// ! DISPLAYDLL_SetPosition !
// +------------------------+
//! Moves the display cursor at a given position.
DLL_EXPORT(void) DISPLAYDLL_SetPosition(LCDS_BYTE x,LCDS_BYTE y) 
{
	static const int line_table[] = {0,64,20,84};
	--x;--y;
	if (y > 3) y = 3;
	int address = line_table[ y] + x;
	pending_commands += std::string("\013") + char( address);
		
}

// +------------------+
// ! DISPLAYDLL_Write !
// +------------------+
//! Writes the given string on the display.
//! The string is displayed at the current cursor position (see DISPLAYDLL_SetPosition).
//! It is assumed that the cursor position has been set so that the string can fit.
DLL_EXPORT(void) DISPLAYDLL_Write(char *str) 
{
	packager.send_string( pending_commands + str);
	pending_commands.clear();
}

// +-----------------+
// ! DISPLAYDLL_Done !
// +-----------------+
DLL_EXPORT(void) DISPLAYDLL_Done(void) {
}

// +--------------------------+
// ! DISPLAYDLL_SetBrightness !
// +--------------------------+
/*DLL_EXPORT(void) DISPLAYDLL_SetBrightness(LCDS_BYTE brightness) {
}

// +-----------------------+
// ! DISPLAYDLL_CustomChar !
// +-----------------------+
/*DLL_EXPORT(void) DISPLAYDLL_CustomChar(LCDS_BYTE chr,LCDS_BYTE *data) {
}

// +----------------------------+
// ! DISPLAYDLL_CustomCharIndex !
// +----------------------------+
DLL_EXPORT(LCDS_BYTE) DISPLAYDLL_CustomCharIndex(LCDS_BYTE index) {
  --index;
  if (0==index) index=8;
  return index;
}


// +--------------------+
// ! DISPLAYDLL_ReadKey !
// +--------------------+
DLL_EXPORT(LCDS_WORD) DISPLAYDLL_ReadKey(void) {
  return 0;
}

// +-------------------------+
// ! DISPLAYDLL_SetBacklight !
// +-------------------------+
DLL_EXPORT(void) DISPLAYDLL_SetBacklight(LCDS_BOOL light_on) {
}

// +------------------------+
// ! DISPLAYDLL_SetContrast !
// +------------------------+
DLL_EXPORT(void) DISPLAYDLL_SetContrast(LCDS_BYTE contrast) {
}

// +------------------------+
// ! DISPLAYDLL_PowerResume !
// +------------------------+
DLL_EXPORT(void) DISPLAYDLL_PowerResume(void) {
}

// +-------------------+
// ! DISPLAYDLL_SetGPO !
// +-------------------+
DLL_EXPORT(void) DISPLAYDLL_SetGPO(LCDS_BYTE gpo,LCDS_BOOL gpo_on) {
}

// +-------------------+
// ! DISPLAYDLL_SetFan !
// +-------------------+
DLL_EXPORT(void) DISPLAYDLL_SetFan(LCDS_BYTE t1,LCDS_BYTE t2) {
}
*/