
//          Copyright Danny Havenith 2009.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)

#if !defined( PACKET_PROTOCOL_HPP)
#define PACKET_PROTOCOL_HPP
#include <boost/foreach.hpp>

/// packet that can be sent to the LCD.
/// whenever a string (a series of characters, including control bytes) is sent, it
/// needs to be chopped up in packets of at most 16 characters.
struct packet
{
	char			address;
	char			size;	// size is in lower nibble
	std::string		payload;// should be 16 characters or smaller.
	unsigned short	checksum;
};

/// Non-virtual-interface on packet destinations.
/// This interface represents devices that can receive packets.
struct packet_sink
{
	template< typename ContainerType>
	void send( const ContainerType &c)
	{
		int count = 0;
		BOOST_FOREACH( const packet &p, c)
		{
			send_packet( p, count++);
		}
	}

private:
	virtual void send_packet( const packet &p, int count) = 0;
};

#endif //PACKET_PROTOCOL_HPP