
//          Copyright Danny Havenith 2009.
// Distributed under the Boost Software License, Version 1.0.
//    (See accompanying file LICENSE_1_0.txt or copy at
//          http://www.boost.org/LICENSE_1_0.txt)


#if !defined( SERIAL_PORT_SINK_HPP)
#define SERIAL_PORT_SINK_HPP

#include <string>
#include <boost/asio/serial_port_service.hpp>
#include <boost/asio/serial_port.hpp>
#include <boost/asio/write.hpp>
#include <boost/assign.hpp>
#include "packet_protocol.hpp"

/// This is a packet_sink that sends the packets to a serial port.
struct serial_port_sink : public packet_sink
{
	serial_port_sink( const std::string &portname)
		: port( service, portname)
	{
		// default serial port options. only the baud rate is non-default..
		port.set_option( boost::asio::serial_port_base::baud_rate( 2400));
		port.set_option( boost::asio::serial_port_base::parity( ));
		port.set_option( boost::asio::serial_port_base::flow_control( ));
		port.set_option( boost::asio::serial_port_base::stop_bits( ));
		port.set_option( boost::asio::serial_port_base::character_size( 8));
	}

private:

	boost::asio::io_service service;
	boost::asio::serial_port port;

	//
	// this implementation of the send_packet function will send the packet 
	// through the serial port.
	//
	virtual void send_packet( const packet &p, int count)
	{
        using boost::asio::write;
		using boost::asio::buffer;
		using boost::assign::list_of;

		// to make sure the checksum bytes are in the right byte order.
		const unsigned char checksum[] = { p.checksum >> 8, p.checksum & 0xFF};

		// the preamble can be anything that ends in 0x00, 0x55
		// followed by the device address and the packet size.
		if (count)
		{
            // this is not the first packet, we can use a short preamble
			const unsigned char preamble[] = { 0x00, 0x55, p.address, p.size };
			write( 
				port, list_of
				( buffer( preamble))
				( buffer( p.payload))
				( buffer( checksum))
				);
		}
		else
		{
            // this is the first packet in a string of packets. Send a slightly larger preamble.
			const unsigned char preamble[] = { 0xff, 0x00, 0x55, p.address, p.size};
			write( 
				port, list_of
				( buffer( preamble))
				( buffer( p.payload))
				( buffer( checksum))
				);
		}

	}

};

#endif //SERIAL_PORT_SINK_HPP
